package gregtech.api.enums;

import gregtech.api.GregTech_API;
import gregtech.api.enums.TC_Aspects.TC_AspectStack;
import gregtech.api.interfaces.IOreRecipeRegistrator;
import gregtech.api.objects.MaterialStack;
import gregtech.api.objects.OrePrefixMaterialData;
import gregtech.api.util.GT_Utility;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

import net.minecraft.item.ItemStack;

public enum OrePrefixes {
	@Deprecated pulp		(""								, ""								, false, false, false, false, false, false,										-1, 64),
	@Deprecated leaves		(""								, ""								, false, false, false, false, false, false,										-1, 64),
	@Deprecated sapling		(""								, ""								, false, false, false, false, false, false,										-1, 64),
	@Deprecated itemDust	(""								, ""								, false, false, false, false, false, false,										-1, 64),
	
	oreBlackgranite			("Granite "						, " Ore"							,  true,  true, false, false, false,  true,										-1, 64), // In case of an End-Ores Mod. Ore -> Material is a Oneway Operation!
	oreRedgranite			("Granite "						, " Ore"							,  true,  true, false, false, false,  true,										-1, 64), // In case of an End-Ores Mod. Ore -> Material is a Oneway Operation!
	oreNetherrack			("Nether "						, " Ore"							,  true,  true, false, false, false,  true,										-1, 64), // Prefix of the Nether-Ores Mod. Causes Ores to double. Ore -> Material is a Oneway Operation!
	oreNether				("Nether "						, " Ore"							,  true,  true, false, false, false,  true,										-1, 64), // Prefix of the Nether-Ores Mod. Causes Ores to double. Ore -> Material is a Oneway Operation!
	@Deprecated denseore	(""								, ""								, false, false, false, false, false,  true,										-1, 64),
	oreDense				("Dense "						, " Ore"							,  true,  true, false, false, false,  true,										-1, 64), // Prefix of the Dense-Ores Mod. Causes Ores to double. Ore -> Material is a Oneway Operation!
	oreRich					("Rich "						, " Ore"							,  true,  true, false, false, false,  true,										-1, 64), // Prefix of TFC
	oreNormal				("Normal "						, " Ore"							,  true,  true, false, false, false,  true,										-1, 64), // Prefix of TFC
	oreSmall				("Small "						, " Ore"							,  true,  true, false, false, false,  true,										-1, 64), // Prefix of Railcraft.
	orePoor					("Poor "						, " Ore"							,  true,  true, false, false, false,  true,										-1, 64), // Prefix of Railcraft.
	oreEndstone				("End "							, " Ore"							,  true,  true, false, false, false,  true,										-1, 64), // In case of an End-Ores Mod. Ore -> Material is a Oneway Operation!
	oreEnd					("End "							, " Ore"							,  true,  true, false, false, false,  true,										-1, 64), // In case of an End-Ores Mod. Ore -> Material is a Oneway Operation!
	@Deprecated oreGem		(""								, ""								, false, false, false, false, false,  true,										-1, 64),
	ore						(""								, " Ore"							,  true,  true, false, false, false,  true,										-1, 64), // Regular Ore Prefix. Ore -> Material is a Oneway Operation! Introduced by Eloraam
	crushedCentrifuged		("Centrifuged "					, " Ore"							,  true,  true, false, false, false, false,										-1, 64), 
	crushedPurified			("Purified "					, " Ore"							,  true,  true, false, false, false, false,										-1, 64), 
	crushed					("Crushed "						, " Ore"							,  true,  true, false, false, false, false,										-1, 64), 
	shard					(""								, ""								,  true,  true, false, false, false, false,										-1, 64),
	clump					(""								, ""								,  true,  true, false, false, false, false,										-1, 64),
	reduced					(""								, ""								,  true,  true, false, false, false, false,										-1, 64),
	crystalline				(""								, ""								,  true,  true, false, false, false, false,										-1, 64),
	cleanGravel				(""								, ""								,  true,  true, false, false, false, false,										-1, 64),
	dirtyGravel				(""								, ""								,  true,  true, false, false, false, false,										-1, 64),
	ingotQuintuple			("Quintuple "					, " Ingot"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 5, 12), // A quintuple Ingot.
	ingotQuadruple			("Quadruple "					, " Ingot"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 4, 16), // A quadruple Ingot.
	@Deprecated ingotQuad	(""								, ""								, false, false, false, false, false, false,										-1, 16),
	ingotTriple				("Triple "						, " Ingot"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 3, 21), // A triple Ingot.
	ingotDouble				("Double "						, " Ingot"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 2, 32), // A double Ingot. Introduced by TerraFirmaCraft
	ingotHot				("Hot "							, " Ingot"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 16), // A hot Ingot, which has to be cooled down by a Vacuum Freezer.
	ingot					(""								, " Ingot"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 64), // A regular Ingot. Introduced by Eloraam
	gemChipped				("Chipped "						, ""								,  true,  true,  true, false, false, false, GregTech_API.MATERIAL_UNIT /		 4, 64), // A regular Gem worth one small Dust. Introduced by TerraFirmaCraft
	gemFlawed				("Flawed "						, ""								,  true,  true,  true, false, false, false, GregTech_API.MATERIAL_UNIT /		 2, 64), // A regular Gem worth two small Dusts. Introduced by TerraFirmaCraft
	gemFlawless				("Flawless "					, ""								,  true,  true,  true, false, false, false, GregTech_API.MATERIAL_UNIT *		 2, 32), // A regular Gem worth two Dusts. Introduced by TerraFirmaCraft
	gemExquisite			("Exquisite "					, ""								,  true,  true,  true, false, false, false, GregTech_API.MATERIAL_UNIT *		 4, 16), // A regular Gem worth four Dusts. Introduced by TerraFirmaCraft
	gem						(""								, ""								,  true,  true,  true, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 64), // A regular Gem worth one Dust. Introduced by Eloraam
	@Deprecated dustDirty	(""								, ""								, false, false, false, false, false, false,										-1, 64),
	dustTiny				("Tiny Pile of "				, " Dust"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT /		 9, 64), // 1/9th of a Dust.
	dustSmall				("Small Pile of "				, " Dust"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT /		 4, 64), // 1/4th of a Dust.
	dustImpure				("Impure Pile of "				, " Dust"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 64), // Dust with impurities. 1 Unit of Main Material and 1/9 - 1/4 Unit of secondary Material
	dustRefined				("Refined Pile of "				, " Dust"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 64),
	dustPure				("Purified Pile of "			, " Dust"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 64), // Dust without impurities.
	dust					(""								, " Dust"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 64), // Pure Dust worth of one Ingot or Gem. Introduced by Alblaka.
	nugget					(""								, " Nugget"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT /		 9, 64), // A Nugget. Introduced by Eloraam
	plateAlloy				(""								, ""								,  true, false, false, false, false, false,										-1, 64), // Special Alloys have this prefix.
	plateSteamcraft			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	plateDense				("Dense "						, " Plate"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 9,  8), // 9 Plates combined in one Item.
	plateQuintuple			("Quintuple "					, " Plate"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 5, 12),
	plateQuadruple			("Quadruple "					, " Plate"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 4, 16),
	@Deprecated plateQuad	(""								, ""								, false, false, false, false, false, false,										-1, 16),
	plateTriple				("Triple "						, " Plate"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 3, 21),
	plateDouble				("Double "						, " Plate"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 2, 32),
	plate					(""								, " Plate"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 64), // Regular Plate made of one Ingot/Dust. Introduced by Calclavia
	foil					(""								, " Foil"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT /		 4, 64), // Foil made of 1/4 Ingot/Dust.
	glass					(""								, ""								, false, false,  true, false, false,  true,										-1, 64),
	paneGlass				(""								, ""								, false, false,  true, false, false,  true,										-1, 64),
	blockGlass				(""								, ""								, false, false,  true, false, false,  true,										-1, 64),
	blockWool				(""								, ""								, false, false,  true, false, false,  true,										-1, 64),
	block_					(""								, ""								, false, false, false, false, false,  true,										-1, 64), // IGNORE
	block					("Block of "					, ""								,  true,  true, false, false, false,  true, GregTech_API.MATERIAL_UNIT *		 9, 64), // Storage Block consisting out of 9 Ingots/Gems/Dusts. Introduced by CovertJaguar
	stick					(""								, " Rod"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT /		 2, 64), // Stick made of half an Ingot. Introduced by Eloraam
	lens					(""								, " Lens"							,  true,  true, false, false, false, false,(GregTech_API.MATERIAL_UNIT * 3)   / 4, 64), // 3/4 of a Plate or Gem used to shape a Lense. Normally only used on Transparent Materials.
	round					(""								, " Round"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT /		 9, 64), // consisting out of one Nugget.
	bolt					(""								, " Bolt"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT /		 8, 64), // consisting out of 1/8 Ingot or 1/4 Stick.
	screw					(""								, " Screw"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT /		 9, 64), // consisting out of a Bolt.
	ring					(""								, " Ring"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT /		 4, 64), // consisting out of 1/2 Stick.
	crateGtDust				("Crate of "					, " Dust"							,  true,  true, false,  true, false, false,										-1, 64), // consisting out of 16 Dusts.
	crateGtPlate			("Crate of "					, " Plate"							,  true,  true, false,  true, false, false,										-1, 64), // consisting out of 16 Plates.
	crateGtIngot			("Crate of "					, " Ingot"							,  true,  true, false,  true, false, false,										-1, 64), // consisting out of 16 Ingots.
	crateGtGem				("Crate of "					, " Gem"							,  true,  true, false,  true, false, false,										-1, 64), // consisting out of 16 Gems.
	cellPlasma				(""								, " Plasma Cell"					,  true,  true,  true,  true, false, false, GregTech_API.MATERIAL_UNIT *		 1, 64), // Hot Cell full of Plasma, which can be used in the Plasma Generator.
	cell					(""								, " Cell"							,  true,  true,  true,  true, false, false, GregTech_API.MATERIAL_UNIT *		 1, 64), // Regular Gas/Fluid Cell. Introduced by Calclavia
	bucket					(""								, " Bucket"							,  true,  true,  true,  true, false, false, GregTech_API.MATERIAL_UNIT *		 1, 16), // A vanilla Iron Bucket filled with the Material.
	bottle					(""								, " Bottle"							,  true,  true,  true,  true, false, false, 									-1, 16), // Glas Bottle containing a Fluid.
	capsule					(""								, " Capsule"						, false,  true,  true,  true, false, false, GregTech_API.MATERIAL_UNIT *		 1, 16),
	crystal					(""								, " Crystal"						, false,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 64),
	craftingTool			(""								, ""								, false, false, false, false, false, false,										-1, 64), // Special Prefix used mainly for the Crafting Handler.
	crafting				(""								, ""								, false, false, false, false, false, false,										-1, 64), // Special Prefix used mainly for the Crafting Handler.
	craft					(""								, ""								, false, false, false, false, false, false,										-1, 64), // Special Prefix used mainly for the Crafting Handler.
	log						(""								, ""								, false, false, false, false, false,  true,										-1, 64), // Prefix used for Logs. Usually as "logWood". Introduced by Eloraam
	slab					(""								, ""								, false, false, false, false, false,  true,										-1, 64), // Prefix used for Slabs. Usually as "slabWood" or "slabStone". Introduced by SirSengir
	stair					(""								, ""								, false, false, false, false, false,  true,										-1, 64), // Prefix used for Stairs. Usually as "stairWood" or "stairStone". Introduced by SirSengir
	fence					(""								, ""								, false, false, false, false, false, false,										-1, 64), // Prefix used for Fences. Usually as "fenceWood". Introduced by Forge
	plank					(""								, ""								, false, false, false, false, false,  true, 									-1, 64), // Prefix for Planks. Usually "plankWood". Introduced by Eloraam
	treeSapling				(""								, ""								, false, false,  true, false, false,  true,										-1, 64), // Prefix for Saplings.
	treeLeaves				(""								, ""								, false, false,  true, false, false,  true,										-1, 64), // Prefix for Leaves.
	tree					(""								, ""								, false, false, false, false, false, false,										-1, 64), // Prefix for Tree Parts.
	stoneCobble				(""								, ""								, false, false,  true, false, false,  true,										-1, 64), // Cobblestone Prefix for all Cobblestones.
	stoneSmooth				(""								, ""								, false, false,  true, false, false,  true,										-1, 64), // Smoothstone Prefix.
	stoneMossyBricks		(""								, ""								, false, false,  true, false, false,  true,										-1, 64), // Mossy Stone Bricks.
	stoneMossy				(""								, ""								, false, false,  true, false, false,  true,										-1, 64), // Mossy Cobble.
	@Deprecated stoneBricksMossy(""							, ""								, false, false, false, false, false,  true,										-1, 64),
	stoneBricks				(""								, ""								, false, false,  true, false, false,  true,										-1, 64), // Stone Bricks.
	@Deprecated stoneBrick	(""								, ""								, false, false, false, false, false,  true,										-1, 64),
	stoneCracked			(""								, ""								, false, false,  true, false, false,  true,										-1, 64), // Cracked Bricks.
	stoneChiseled			(""								, ""								, false, false,  true, false, false,  true,										-1, 64), // Chiseled Stone.
	stone					(""								, ""								, false,  true,  true, false,  true,  true,										-1, 64), // Prefix to determine which kind of Rock this is.
	cobblestone				(""								, ""								, false,  true,  true, false, false,  true,										-1, 64),
	rock					(""								, ""								, false,  true,  true, false,  true,  true,										-1, 64), // Prefix to determine which kind of Rock this is.
	record					(""								, ""								, false, false,  true, false, false, false,										-1,  1),
	rubble					(""								, ""								,  true,  true,  true, false, false, false,										-1, 64),
	scraps					(""								, ""								,  true,  true, false, false, false, false,										-1, 64),
	scrap					(""								, ""								, false, false, false, false, false, false,										-1, 64),
	item_					(""								, ""								, false, false, false, false, false, false,										-1, 64), // IGNORE
	item					(""								, ""								, false, false, false, false, false, false,										-1, 64), // Random Item. Introduced by Alblaka
	book					(""								, ""								, false, false, false, false, false, false,										-1, 64), // Used for Books of any kind.
	paper					(""								, ""								, false, false, false, false, false, false,										-1, 64), // Used for Papers of any kind.
	dye						(""								, ""								, false, false,  true, false, false, false,										-1, 64), // Used for the 16 dyes. Introduced by Eloraam
	stainedClay				(""								, ""								, false, false,  true, false, false,  true,										-1, 64), // Used for the 16 colors of Stained Clay. Introduced by Forge
	armorHelmet				(""								, ""								, false,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 5,  1), // vanilly Helmet
	armorChestplate			(""								, ""								, false,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 8,  1), // vanilly Chestplate
	armorLeggins			(""								, ""								, false,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 7,  1), // vanilly Pants
	armorBoots				(""								, ""								, false,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 4,  1), // vanilly Boots
	armor					(""								, ""								, false, false, false, false, false, false, 									-1,  1),
	toolHeadSword			(""								, " Sword Blade"					,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 2, 16), // consisting out of 2 Ingots.
	toolHeadPickaxe			(""								, " Pickaxe Head"					,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 3, 16), // consisting out of 3 Ingots.
	toolHeadShovel			(""								, " Shovel Head"					,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 16), // consisting out of 1 Ingots.
	toolHeadUniversalSpade	(""								, " Universal Spade Head"			,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 16), // consisting out of 1 Ingots.
	toolHeadAxe				(""								, " Axe Head"						,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 3, 16), // consisting out of 3 Ingots.
	toolHeadHoe				(""								, " Hoe Head"						,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 2, 16), // consisting out of 2 Ingots.
	toolHeadSense			(""								, " Sense Blade"					,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 3, 16), // consisting out of 3 Ingots.
	toolHeadFile			(""								, " File Head"						,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 2, 16), // consisting out of 2 Ingots.
	toolHeadHammer			(""								, " Hammer Head"					,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 6, 16), // consisting out of 6 Ingots.
	toolHeadPlow			(""								, " Plow Head"						,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 4, 16), // consisting out of 4 Ingots.
	toolHeadSaw				(""								, " Saw Blade"						,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 2, 16), // consisting out of 2 Ingots.
	toolHeadScrewdriver		(""								, " Screwdriver Tip"				,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 16), // consisting out of 1 Ingots.
	toolHeadDrill			(""								, " Drill Tip"						,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 4, 16), // consisting out of 4 Ingots.
	toolHeadChainsaw		(""								, " Chainsaw Tip"					,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 2, 16), // consisting out of 2 Ingots.
	toolHeadWrench			(""								, " Wrench Tip"						,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 4, 16), // consisting out of 4 Ingots.
	toolSword				(""								, ""								, false,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 2,  1), // vanilly Sword
	toolPickaxe				(""								, ""								, false,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 3,  1), // vanilly Pickaxe
	toolShovel				(""								, ""								, false,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1,  1), // vanilly Shovel
	toolAxe					(""								, ""								, false,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 3,  1), // vanilly Axe
	toolHoe					(""								, ""								, false,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 2,  1), // vanilly Hoe
	toolShears				(""								, ""								, false,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 2,  1), // vanilly Shears
	tool					(""								, ""								, false, false, false, false, false, false, 									-1,  1), // toolPot, toolSkillet, toolSaucepan, toolBakeware, toolCuttingboard, toolMortarandpestle, toolMixingbowl, toolJuicer
	frameGt					(""								, ""								,  true,  true, false, false,  true, false, GregTech_API.MATERIAL_UNIT *		 2, 64),
	pipeTiny				("Tiny "						, " Pipe"							,  true,  true, false, false,  true, false, GregTech_API.MATERIAL_UNIT /		 2, 64),
	pipeSmall				("Small "						, " Pipe"							,  true,  true, false, false,  true, false, GregTech_API.MATERIAL_UNIT *		 1, 64),
	pipeMedium				("Medium "						, " Pipe"							,  true,  true, false, false,  true, false, GregTech_API.MATERIAL_UNIT *		 3, 64),
	pipeLarge				("Large "						, " Pipe"							,  true,  true, false, false,  true, false, GregTech_API.MATERIAL_UNIT *		 6, 64),
	pipeHuge				("Huge "						, " Pipe"							,  true,  true, false, false,  true, false, GregTech_API.MATERIAL_UNIT *		12, 64),
	pipeRestrictiveTiny		("Tiny Restrictive "			, " Pipe"							,  true,  true, false, false,  true, false, GregTech_API.MATERIAL_UNIT /		 2, 64),
	pipeRestrictiveSmall	("Small Restrictive "			, " Pipe"							,  true,  true, false, false,  true, false, GregTech_API.MATERIAL_UNIT *		 1, 64),
	pipeRestrictiveMedium	("Medium Restrictive "			, " Pipe"							,  true,  true, false, false,  true, false, GregTech_API.MATERIAL_UNIT *		 3, 64),
	pipeRestrictiveLarge	("Large Restrictive "			, " Pipe"							,  true,  true, false, false,  true, false, GregTech_API.MATERIAL_UNIT *		 6, 64),
	pipeRestrictiveHuge		("Huge Restrictive "			, " Pipe"							,  true,  true, false, false,  true, false, GregTech_API.MATERIAL_UNIT *		12, 64),
	pipe					(""								, " Pipe"							, false, false, false, false, false, false, 									-1, 64),
	gearGt					(""								, " Gear"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 4, 16), // Introduced by me because BuildCraft has ruined the gear Prefix...
	wireGt16				("16x "							, " Wire"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 8, 64),
	wireGt12				("12x "							, " Wire"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 6, 64),
	wireGt08				("8x "							, " Wire"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 4, 64),
	wireGt04				("4x "							, " Wire"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 2, 64),
	wireGt02				("2x "							, " Wire"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 64),
	wireGt01				("1x "							, " Wire"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT /		 2, 64),
	cableGt12				("12x "							, " Cable"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 6, 64),
	cableGt08				("8x "							, " Cable"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 4, 64),
	cableGt04				("4x "							, " Cable"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 2, 64),
	cableGt02				("2x "							, " Cable"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT *		 1, 64),
	cableGt01				("1x "							, " Cable"							,  true,  true, false, false, false, false, GregTech_API.MATERIAL_UNIT /		 2, 64),
	
	/* Electric Components.
	 * 
	 * usual Materials for this are:
	 * Primitive (Tier 1)
	 * Basic (Tier 2) as used by UE as well : IC2 Circuit and RE-Battery
	 * Good (Tier 3)
	 * Advanced (Tier 4) as used by UE as well : Advanced Circuit, Advanced Battery and Lithium Battery
	 * Data (Tier 5) : Data Storage Circuit
	 * Elite (Tier 6) as used by UE as well : Energy Crystal and Data Control Circuit
	 * Master (Tier 7) : Energy Flow Circuit and Lapotron Crystal
	 * Ultimate (Tier 8) : Data Orb and Lapotronic Energy Orb
	 * Infinite (Cheaty)
	 */
	batterySingleuse	(""								, ""								, false,  true, false, false, false, false,										-1, 64),
	battery				(""								, ""								, false,  true, false, false, false, false,										-1, 64), // Introduced by Calclavia
	circuit				(""								, ""								,  true,  true, false, false, false, false,										-1, 64), // Introduced by Calclavia
	computer			(""								, ""								,  true,  true, false, false,  true, false,										-1, 64), // A whole Computer. "computerMaster" = ComputerCube
	
	// random known prefixes without special abilities.
	plating				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	dinosaur			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	travelgear			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	bauble				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	cluster				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	grafter				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	scoop				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	frame				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	tome				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	junk				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	bee					(""								, ""								, false, false, false, false, false, false,										-1, 64),
	rod					(""								, ""								, false, false, false, false, false, false,										-1, 64),
	dirt				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	sand				(""								, ""								, false, false, true , false, false, false,										-1, 64),
	grass				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	gravel				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	mushroom			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	wood				(""								, ""								, false, false, false, false, false, false,										-1, 64), // Introduced by Eloraam
	drop				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	fuel				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	panel				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	brick				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	chunk				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	wire				(""								, ""								, false, false, false, false, true , false,										-1, 64),
	seed				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	reed				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	sheetDouble			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	sheet				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	crop				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	plant				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	coin				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	lumar				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	ground				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	cable				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	component			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	wax					(""								, ""								, false, false, false, false, false, false,										-1, 64),
	wall				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	tube				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	list				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	food				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	gear				(""								, ""								, false, false, false, false, false, false,										-1, 64), // Introduced by SirSengir
	coral				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	flower				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	storage				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	material			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	plasma				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	element				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	molecule			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	wafer				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	orb					(""								, ""								, false, false, false, false, false, false,										-1, 64),
	handle				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	blade				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	head				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	motor				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	bit					(""								, ""								, false, false, false, false, false, false,										-1, 64),
	shears				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	turbine				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	compressed			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	fertilizer			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	chest				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	raw					(""								, ""								, false, false, false, false, false, false,										-1, 64),
	stainedGlass		(""								, ""								, false, false, false, false, false, false,										-1, 64),
	mystic				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	mana				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	rune				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	petal				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	pearl				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	powder				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	soulsand			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	obsidian			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	glowstone			(""								, ""								, false, false, false, false, false, false,										-1, 64),
	beans				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	br					(""								, ""								, false, false, false, false, false, false,										-1, 64),
	essence				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	alloy				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	cooking				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	elven				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	reactor				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	mffs				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	projred				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	ganys				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	bars				(""								, ""								, false, false, false, false, false, false,										-1, 64),
	bar					(""								, ""								, false, false, false, false, false, false,										-1, 64);
	
	
	static {
		pulp.mPrefixInto = dust;
		oreGem.mPrefixInto = ore;
		leaves.mPrefixInto = treeLeaves;
		sapling.mPrefixInto = treeSapling;
		itemDust.mPrefixInto = dust;
		dustDirty.mPrefixInto = dustImpure;
		denseore.mPrefixInto = oreDense;
		ingotQuad.mPrefixInto = ingotQuadruple;
		plateQuad.mPrefixInto = plateQuadruple;
		stoneBrick.mPrefixInto = stoneBricks;
		stoneBricksMossy.mPrefixInto = stoneMossyBricks;
		
		ingotHot.mHeatDamage = 3.0F;
		cellPlasma.mHeatDamage = 6.0F;
		
		block.ignoreMaterials(Materials.Ice, Materials.Snow, Materials.Concrete, Materials.Glass, Materials.Glowstone, Materials.DarkIron, Materials.Marble, Materials.Quartz, Materials.CertusQuartz, Materials.Limestone);
		ingot.ignoreMaterials(Materials.Brick, Materials.NetherBrick);
		
		dust.addFamiliarPrefix(dustTiny);
		dust.addFamiliarPrefix(dustSmall);
		dustTiny.addFamiliarPrefix(dust);
		dustTiny.addFamiliarPrefix(dustSmall);
		dustSmall.addFamiliarPrefix(dust);
		dustSmall.addFamiliarPrefix(dustTiny);
		
		ingot.addFamiliarPrefix(nugget);
		nugget.addFamiliarPrefix(ingot);
		
		for (OrePrefixes tPrefix1 : values()) if (tPrefix1.name().startsWith("ore"		)) for (OrePrefixes tPrefix2 : values()) if (tPrefix2.name().startsWith("ore"		)) tPrefix1.addFamiliarPrefix(tPrefix2);
		for (OrePrefixes tPrefix1 : values()) if (tPrefix1.name().startsWith("pipe"		)) for (OrePrefixes tPrefix2 : values()) if (tPrefix2.name().startsWith("pipe"		)) tPrefix1.addFamiliarPrefix(tPrefix2);
		for (OrePrefixes tPrefix1 : values()) if (tPrefix1.name().startsWith("wireGt"	)) for (OrePrefixes tPrefix2 : values()) if (tPrefix2.name().startsWith("wireGt"	)) tPrefix1.addFamiliarPrefix(tPrefix2);
		for (OrePrefixes tPrefix1 : values()) if (tPrefix1.name().startsWith("cableGt"	)) for (OrePrefixes tPrefix2 : values()) if (tPrefix2.name().startsWith("cableGt"	)) tPrefix1.addFamiliarPrefix(tPrefix2);
		
		// These are only the important ones.
		gem.mNotGeneratedItems.add(Materials.Coal);
		gem.mNotGeneratedItems.add(Materials.Charcoal);
		gem.mNotGeneratedItems.add(Materials.NetherStar);
		gem.mNotGeneratedItems.add(Materials.Diamond);
		gem.mNotGeneratedItems.add(Materials.Emerald);
		gem.mNotGeneratedItems.add(Materials.NetherQuartz);
		gem.mNotGeneratedItems.add(Materials.EnderPearl);
		gem.mNotGeneratedItems.add(Materials.EnderEye);
		gem.mNotGeneratedItems.add(Materials.Flint);
		gem.mNotGeneratedItems.add(Materials.Lapis);
		dust.mNotGeneratedItems.add(Materials.Bone);
		dust.mNotGeneratedItems.add(Materials.Redstone);
		dust.mNotGeneratedItems.add(Materials.Glowstone);
		dust.mNotGeneratedItems.add(Materials.Gunpowder);
		dust.mNotGeneratedItems.add(Materials.Sugar);
		dust.mNotGeneratedItems.add(Materials.Blaze);
		stick.mNotGeneratedItems.add(Materials.Wood);
		stick.mNotGeneratedItems.add(Materials.Bone);
		stick.mNotGeneratedItems.add(Materials.Blaze);
		ingot.mNotGeneratedItems.add(Materials.Iron);
		ingot.mNotGeneratedItems.add(Materials.Gold);
		ingot.mNotGeneratedItems.add(Materials.Brick);
		ingot.mNotGeneratedItems.add(Materials.BrickNether);
		ingot.mNotGeneratedItems.add(Materials.WoodSealed);
		ingot.mNotGeneratedItems.add(Materials.Wood);
		nugget.mNotGeneratedItems.add(Materials.Gold);
		plate.mNotGeneratedItems.add(Materials.Paper);
		cell.mNotGeneratedItems.add(Materials.Empty);
		cell.mNotGeneratedItems.add(Materials.Water);
		cell.mNotGeneratedItems.add(Materials.Lava);
		cell.mNotGeneratedItems.add(Materials.ConstructionFoam);
		cell.mNotGeneratedItems.add(Materials.UUMatter);
		cell.mNotGeneratedItems.add(Materials.BioFuel);
		cell.mNotGeneratedItems.add(Materials.CoalFuel);
		cellPlasma.mNotGeneratedItems.add(Materials.Empty);
		bucket.mNotGeneratedItems.add(Materials.Empty);
		bucket.mNotGeneratedItems.add(Materials.Lava);
		bucket.mNotGeneratedItems.add(Materials.Milk);
		bucket.mNotGeneratedItems.add(Materials.Water);
		bottle.mNotGeneratedItems.add(Materials.Empty);
		bottle.mNotGeneratedItems.add(Materials.Water);
		bottle.mNotGeneratedItems.add(Materials.Milk);
		block.mNotGeneratedItems.add(Materials.Iron);
		block.mNotGeneratedItems.add(Materials.Gold);
		block.mNotGeneratedItems.add(Materials.Lapis);
		block.mNotGeneratedItems.add(Materials.Emerald);
		block.mNotGeneratedItems.add(Materials.Redstone);
		block.mNotGeneratedItems.add(Materials.Diamond);
		block.mNotGeneratedItems.add(Materials.Coal);

		pipeRestrictiveTiny.mSecondaryMaterial		= new MaterialStack(Materials.Steel, OrePrefixes.ring.mMaterialAmount * 1);
		pipeRestrictiveSmall.mSecondaryMaterial		= new MaterialStack(Materials.Steel, OrePrefixes.ring.mMaterialAmount * 2);
		pipeRestrictiveMedium.mSecondaryMaterial	= new MaterialStack(Materials.Steel, OrePrefixes.ring.mMaterialAmount * 3);
		pipeRestrictiveLarge.mSecondaryMaterial		= new MaterialStack(Materials.Steel, OrePrefixes.ring.mMaterialAmount * 4);
		pipeRestrictiveHuge.mSecondaryMaterial		= new MaterialStack(Materials.Steel, OrePrefixes.ring.mMaterialAmount * 5);
		cableGt12.mSecondaryMaterial = new MaterialStack(Materials.Rubber, GregTech_API.MATERIAL_UNIT * 4);
		cableGt08.mSecondaryMaterial = new MaterialStack(Materials.Rubber, GregTech_API.MATERIAL_UNIT * 3);
		cableGt04.mSecondaryMaterial = new MaterialStack(Materials.Rubber, GregTech_API.MATERIAL_UNIT * 2);
		cableGt02.mSecondaryMaterial = new MaterialStack(Materials.Rubber, GregTech_API.MATERIAL_UNIT * 1);
		cableGt01.mSecondaryMaterial = new MaterialStack(Materials.Rubber, GregTech_API.MATERIAL_UNIT * 1);
		bucket.mSecondaryMaterial = new MaterialStack(Materials.Iron, GregTech_API.MATERIAL_UNIT * 3);
		cell.mSecondaryMaterial = new MaterialStack(Materials.Tin, GregTech_API.MATERIAL_UNIT / 2);
		cellPlasma.mSecondaryMaterial = new MaterialStack(Materials.Tin, GregTech_API.MATERIAL_UNIT / 2);
		oreRedgranite.mSecondaryMaterial = new MaterialStack(Materials.GraniteRed, GregTech_API.MATERIAL_UNIT * 1);
		oreBlackgranite.mSecondaryMaterial = new MaterialStack(Materials.GraniteBlack, GregTech_API.MATERIAL_UNIT * 1);
		oreNetherrack.mSecondaryMaterial = new MaterialStack(Materials.Netherrack, GregTech_API.MATERIAL_UNIT * 1);
		oreNether.mSecondaryMaterial = new MaterialStack(Materials.Netherrack, GregTech_API.MATERIAL_UNIT * 1);
		oreEndstone.mSecondaryMaterial = new MaterialStack(Materials.Endstone, GregTech_API.MATERIAL_UNIT * 1);
		oreEnd.mSecondaryMaterial = new MaterialStack(Materials.Endstone, GregTech_API.MATERIAL_UNIT * 1);
		oreDense.mSecondaryMaterial = new MaterialStack(Materials.Stone, GregTech_API.MATERIAL_UNIT * 1);
		orePoor.mSecondaryMaterial = new MaterialStack(Materials.Stone, GregTech_API.MATERIAL_UNIT * 2);
		oreSmall.mSecondaryMaterial = new MaterialStack(Materials.Stone, GregTech_API.MATERIAL_UNIT * 2);
		oreNormal.mSecondaryMaterial = new MaterialStack(Materials.Stone, GregTech_API.MATERIAL_UNIT * 2);
		oreRich.mSecondaryMaterial = new MaterialStack(Materials.Stone, GregTech_API.MATERIAL_UNIT * 2);
		ore.mSecondaryMaterial = new MaterialStack(Materials.Stone, GregTech_API.MATERIAL_UNIT * 1);
		crushed.mSecondaryMaterial = new MaterialStack(Materials.Stone, GregTech_API.MATERIAL_UNIT * 1);
		toolHeadDrill.mSecondaryMaterial = new MaterialStack(Materials.Steel, plate.mMaterialAmount * 4);
		toolHeadChainsaw.mSecondaryMaterial = new MaterialStack(Materials.Steel, plate.mMaterialAmount * 4 + ring.mMaterialAmount * 2);
		toolHeadWrench.mSecondaryMaterial = new MaterialStack(Materials.Steel, ring.mMaterialAmount * 1 + screw.mMaterialAmount * 2);
	}
	
	public final ArrayList<ItemStack> mPrefixedItems = new ArrayList<ItemStack>();
	
	public boolean add(ItemStack aStack) {
		if (aStack == null) return false;
		if (!contains(aStack)) mPrefixedItems.add(aStack);
		while (mPrefixedItems.contains(null)) mPrefixedItems.remove(null);
		return true;
	}
	
	public boolean contains(ItemStack aStack) {
		if (aStack == null) return false;
		for (ItemStack tStack : mPrefixedItems) if (GT_Utility.areStacksEqual(aStack, tStack, !tStack.hasTagCompound())) return true;
		return false;
	}
	
	public boolean dontGenerateItem(Materials aMaterial) {
		return mNotGeneratedItems.contains(aMaterial);
	}
	
	public boolean ignoreMaterials(Materials... aMaterials) {
		for (Materials tMaterial : aMaterials) mIgnoredMaterials.add(tMaterial);
		return true;
	}
	
	public boolean isIgnored(Materials aMaterial) {
		return mIgnoredMaterials.contains(aMaterial);
	}

	public boolean addFamiliarPrefix(OrePrefixes aPrefix) {
		if (aPrefix == null || mFamiliarPrefixes.contains(aPrefix) || aPrefix == this) return false;
		return mFamiliarPrefixes.add(aPrefix);
	}
	
	public boolean add(IOreRecipeRegistrator aRegistrator) {
		if (aRegistrator == null) return false;
		return mOreProcessing.add(aRegistrator);
	}
	
	public void processOre(Materials aMaterial, String aOreDictName, String aModName, ItemStack aStack) {
		if (aMaterial != null && (aMaterial != Materials._NULL || mIsSelfReferencing || !mIsMaterialBased) && GT_Utility.isStackValid(aStack)) for (IOreRecipeRegistrator tRegistrator : mOreProcessing) tRegistrator.registerOre(this, aMaterial, aOreDictName, aModName, GT_Utility.copyAmount(1, aStack));
	}
	
	public byte mDefaultStackSize = 64;
	public final String mLocalizedMaterialPre, mLocalizedMaterialPost;
	public final boolean mIsUnificatable, mIsMaterialBased, mIsSelfReferencing, mIsContainer, mDontUnificateActively, mIsUsedForBlocks;
	public MaterialStack mSecondaryMaterial = null;
	public OrePrefixes mPrefixInto = this;
	public final List<TC_AspectStack> mAspects = new ArrayList<TC_AspectStack>();
	public final Collection<OrePrefixes> mFamiliarPrefixes = new HashSet<OrePrefixes>();
	private final Collection<Materials> mNotGeneratedItems = new HashSet<Materials>(), mIgnoredMaterials = new HashSet<Materials>();
	private final ArrayList<IOreRecipeRegistrator> mOreProcessing = new ArrayList<IOreRecipeRegistrator>();
	public float mHeatDamage = 0.0F; // Negative for Frost Damage
	
	/**
	 * Used to determine the amount of Material this Prefix contains.
	 * Multiply or Divide GregTech_API.MATERIAL_UNIT to set the Amounts in comparision to one Ingot.
	 * 0 = Null
	 * Negative = Undefined Amount
	 */
	public final long mMaterialAmount;
	
	private OrePrefixes(String aLocalizedMaterialPre, String aLocalizedMaterialPost, boolean aIsUnificatable, boolean aIsMaterialBased, boolean aIsSelfReferencing, boolean aIsContainer, boolean aDontUnificateActively, boolean aIsUsedForBlocks, long aMaterialAmount, int aDefaultStackSize) {
		mIsUnificatable = aIsUnificatable;
		mIsMaterialBased = aIsMaterialBased;
		mIsSelfReferencing = aIsSelfReferencing;
		mIsContainer = aIsContainer;
		mDontUnificateActively = aDontUnificateActively;
		mIsUsedForBlocks = aIsUsedForBlocks;
		mMaterialAmount = aMaterialAmount;
		mLocalizedMaterialPre = aLocalizedMaterialPre;
		mLocalizedMaterialPost = aLocalizedMaterialPost;
		mDefaultStackSize = (byte)aDefaultStackSize;
		
		if (name().startsWith("ore")) {
			new TC_AspectStack(TC_Aspects.TERRA, 1).addToAspectList(mAspects);
		} else
		if (name().startsWith("wire") || name().startsWith("cable")) {
			new TC_AspectStack(TC_Aspects.ELECTRUM, 1).addToAspectList(mAspects);
		} else
		if (name().startsWith("dust")) {
			new TC_AspectStack(TC_Aspects.PERDITIO, 1).addToAspectList(mAspects);
		} else
		if (name().startsWith("crushed")) {
			new TC_AspectStack(TC_Aspects.PERFODIO, 1).addToAspectList(mAspects);
		} else
		if (name().startsWith("ingot") || name().startsWith("nugget")) {
			new TC_AspectStack(TC_Aspects.METALLUM, 1).addToAspectList(mAspects);
		} else
		if (name().startsWith("armor")) {
			new TC_AspectStack(TC_Aspects.TUTAMEN, 1).addToAspectList(mAspects);
		} else
		if (name().startsWith("stone")) {
			new TC_AspectStack(TC_Aspects.TERRA, 1).addToAspectList(mAspects);
		} else
		if (name().startsWith("pipe")) {
			new TC_AspectStack(TC_Aspects.ITER, 1).addToAspectList(mAspects);
		} else
		if (name().startsWith("gear")) {
			new TC_AspectStack(TC_Aspects.MOTUS, 1).addToAspectList(mAspects);
			new TC_AspectStack(TC_Aspects.MACHINA, 1).addToAspectList(mAspects);
		} else
		if (name().startsWith("frame") || name().startsWith("plate")) {
			new TC_AspectStack(TC_Aspects.FABRICO, 1).addToAspectList(mAspects);
		} else
		if (name().startsWith("tool")) {
			new TC_AspectStack(TC_Aspects.INSTRUMENTUM, 2).addToAspectList(mAspects);
		} else
		if (name().startsWith("gem") || name().startsWith("crystal") || name().startsWith("lens")) {
			new TC_AspectStack(TC_Aspects.VITREUS, 1).addToAspectList(mAspects);
		} else
		if (name().startsWith("crate")) {
			new TC_AspectStack(TC_Aspects.ITER, 2).addToAspectList(mAspects);
		} else
		if (name().startsWith("circuit")) {
			new TC_AspectStack(TC_Aspects.COGNITO, 1).addToAspectList(mAspects);
		} else
		if (name().startsWith("computer")) {
			new TC_AspectStack(TC_Aspects.COGNITO, 4).addToAspectList(mAspects);
		} else
		if (name().startsWith("battery")) {
			new TC_AspectStack(TC_Aspects.ELECTRUM, 1).addToAspectList(mAspects);
		}
	}
	
	public static OrePrefixes getOrePrefix(String aOre) {
		for (OrePrefixes tPrefix : values()) if (aOre.startsWith(tPrefix.toString())) {
			if (tPrefix == oreNether && aOre.equals("oreNetherQuartz")) return ore;
			return tPrefix;
		}
		return null;
	}
	
	public static String stripPrefix(String aOre) {
		for (OrePrefixes tPrefix : values()) {
			if (aOre.startsWith(tPrefix.toString())) {
				return aOre.replaceFirst(tPrefix.toString(), "");
			}
		}
		return aOre;
	}
	
	public static String replacePrefix(String aOre, OrePrefixes aPrefix) {
		for (OrePrefixes tPrefix : values()) {
			if (aOre.startsWith(tPrefix.toString())) {
				return aOre.replaceFirst(tPrefix.toString(), aPrefix.toString());
			}
		}
		return "";
	}
	
	public static OrePrefixes getPrefix(String aPrefixName) {
		return getPrefix(aPrefixName, null);
	}
	
	public static OrePrefixes getPrefix(String aPrefixName, OrePrefixes aReplacement) {
		Object tObject = GT_Utility.getFieldContent(OrePrefixes.class, aPrefixName, false, false);
		if (tObject != null && tObject instanceof OrePrefixes) return (OrePrefixes)tObject;
		return aReplacement;
	}
	
	public Object get(Object aMaterial) {
		if (aMaterial instanceof Materials) return new OrePrefixMaterialData(this, (Materials)aMaterial);
		return name() + aMaterial;
	}
	
	public static Materials getMaterial(String aOre) {
		return Materials.get(stripPrefix(aOre));
	}

	public static Materials getMaterial(String aOre, OrePrefixes aPrefix) {
		return Materials.get(aOre.replaceFirst(aPrefix.toString(), ""));
	}
	
	public static Materials getRealMaterial(String aOre, OrePrefixes aPrefix) {
		return Materials.getRealMaterial(aOre.replaceFirst(aPrefix.toString(), ""));
	}
	
	public static boolean isInstanceOf(String aName, OrePrefixes aPrefix) {
		return aName == null ? false : aName.startsWith(aPrefix.toString());
	}
	
	public static volatile int VERSION = 505;
	
	public String getDefaultLocalNameForItem(Materials aMaterial) {
		// Certain Materials have slightly different Localizations.
		switch (this) {
		case crateGtDust:	return mLocalizedMaterialPre + OrePrefixes.dust.getDefaultLocalNameForItem(aMaterial);
		case crateGtIngot:	return mLocalizedMaterialPre + OrePrefixes.ingot.getDefaultLocalNameForItem(aMaterial);
		case crateGtGem:	return mLocalizedMaterialPre + OrePrefixes.gem.getDefaultLocalNameForItem(aMaterial);
		case crateGtPlate:	return mLocalizedMaterialPre + OrePrefixes.plate.getDefaultLocalNameForItem(aMaterial);
		}
		switch (aMaterial) {
		case Glass:
			if (name().startsWith("gem"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Crystal";
			if (name().startsWith("plate"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Pane";
			break;
		case InfusedAir: case InfusedDull: case InfusedEarth: case InfusedEntropy: case InfusedFire: case InfusedOrder: case InfusedVis: case InfusedWater:
			if (name().startsWith("gem"))				return mLocalizedMaterialPre + "Shard of " + aMaterial.mDefaultLocalName;
			if (name().startsWith("crystal"))			return mLocalizedMaterialPre + "Shard of " + aMaterial.mDefaultLocalName;
			if (name().startsWith("plate"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Crystal Plate";
			if (name().startsWith("dust"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Crystal Powder";
			if (this == OrePrefixes.crushedCentrifuged)	return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Crystals";
			if (this == OrePrefixes.crushedPurified)	return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Crystals";
			if (this == OrePrefixes.crushed)			return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Crystals";
			break;
		case Wheat:
			if (name().startsWith("dust"))				return mLocalizedMaterialPre + "Flour";
			break;
		case Ice:
			if (name().startsWith("dust"))				return mLocalizedMaterialPre + "Crushed Ice";
			break;
		case Wood: case WoodSealed:
			if (name().startsWith("bolt"))				return "Short " + aMaterial.mDefaultLocalName + " Stick";
			if (name().startsWith("stick"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Stick";
			if (name().startsWith("dust"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Pulp";
			if (name().startsWith("nugget"))			return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Chip";
			if (name().startsWith("plate"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Plank";
			break;
		case Plastic: case Rubber:
			if (name().startsWith("dust"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Pulp";
			if (name().startsWith("plate"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Sheet";
			if (name().startsWith("ingot"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Bar";
			if (name().startsWith("nugget"))			return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Chip";
			if (name().startsWith("foil"))				return "Thin " + aMaterial.mDefaultLocalName + " Sheet";
			break;
		case SteelLeaf:
			if (name().startsWith("ingot"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName;
			break;
		case Blaze: case Milk: case Cocoa: case Chocolate: case Coffee: case Chili: case Cheese: case Snow:
			if (name().startsWith("dust"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + " Powder";
			break;
		case Paper:
			if (name().startsWith("dust"))				return mLocalizedMaterialPre + "Chad";
			if (this == OrePrefixes.plate)				return "Sheet of Paper";
			if (this == OrePrefixes.plateDouble)		return "Paperboard";
			if (this == OrePrefixes.plateTriple)		return "Carton";
			if (this == OrePrefixes.plateQuadruple)		return "Cardboard";
			if (this == OrePrefixes.plateQuintuple)		return "Thick Cardboard";
			if (this == OrePrefixes.plateDense)			return "Strong Cardboard";
			break;
		case MeatRaw:
			if (name().startsWith("dust"))				return mLocalizedMaterialPre + "Mince Meat";
			break;
		case MeatCooked:
			if (name().startsWith("dust"))				return mLocalizedMaterialPre + "Cooked Mince Meat";
			break;
		case Ash: case DarkAsh: case Gunpowder: case Sugar: case Salt: case RockSalt: case VolcanicAsh: case RareEarth:
			if (name().startsWith("dust"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName;
			break;
		case Vermiculite: case Bentonite: case Kaolinite: case Talc: case BasalticMineralSand: case GraniticMineralSand: case CassiteriteSand: case GarnetSand: case QuartzSand: case Pitchblende: case FullersEarth:
			if (name().startsWith("dust"))				return mLocalizedMaterialPre + aMaterial.mDefaultLocalName;
			if (this == OrePrefixes.crushedCentrifuged)	return mLocalizedMaterialPre + aMaterial.mDefaultLocalName;
			if (this == OrePrefixes.crushedPurified)	return mLocalizedMaterialPre + aMaterial.mDefaultLocalName;
			if (this == OrePrefixes.crushed)			return "Ground " + aMaterial.mDefaultLocalName;
			break;
		}
		// Use Standard Localization
		return mLocalizedMaterialPre + aMaterial.mDefaultLocalName + mLocalizedMaterialPost;
	}
}